/*
 * ExpressionStateDictionary.cs --
 *
 * Copyright (c) 2007-2012 by Joe Mistachkin.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * RCS: @(#) $Id: $
 */

using System.Collections.Generic;
using Eagle._Attributes;
using Eagle._Components.Private;
using Eagle._Components.Public;
using Eagle._Constants;
using Eagle._Interfaces.Public;

namespace Eagle._Containers.Public
{
    [ObjectId("c6237200-b1fe-429d-8ac3-9adbd45644f5")]
    public sealed class ExpressionStateDictionary :
#if CACHE_DICTIONARY
        CacheDictionary<string, IExpressionState>
#else
        Dictionary<string, IExpressionState>
#endif
    {
        #region Private Data
#if CACHE_STATISTICS
        internal int[] cacheCounts = new int[(int)CacheCountType.SizeOf]; // WARNING: CACHE USE ONLY.
#endif
        #endregion

        ///////////////////////////////////////////////////////////////////////////////////////////////

        #region Public Constructors
        public ExpressionStateDictionary()
            : base()
        {
            // do nothing.
        }

        ///////////////////////////////////////////////////////////////////////////////////////////////

        public ExpressionStateDictionary(
            IDictionary<string, IExpressionState> dictionary
            )
            : base(dictionary)
        {
            // do nothing.
        }

        ///////////////////////////////////////////////////////////////////////////////////////////////

        public ExpressionStateDictionary(
            IEnumerable<IExpressionState> collection
            )
            : this()
        {
            foreach (IExpressionState item in collection)
                this.Add(item.ParseState.Text, item);
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////////////////////////////

        #region Public Methods
#if CACHE_STATISTICS
        public string CacheToString()
        {
            return StringList.MakeList(
                "count", this.Count, FormatOps.CacheCounts(cacheCounts));
        }
#endif

        ///////////////////////////////////////////////////////////////////////////////////////////////

        public string ToString(
            string pattern,
            bool noCase
            )
        {
            StringList list = new StringList(this.Keys);

            return GenericOps<string>.ListToString(list, Index.Invalid, Index.Invalid,
                ToStringFlags.None, Characters.Space.ToString(), pattern, noCase);
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////////////////////////////

        #region System.Object Overrides
        public override string ToString()
        {
            return ToString(null, false);
        }
        #endregion
    }
}
