/*
 * Debugger.cs --
 *
 * Copyright (c) 2007-2012 by Joe Mistachkin.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * RCS: @(#) $Id: $
 */

using Eagle._Attributes;
using Eagle._Components.Public;
using Eagle._Containers.Private;
using Eagle._Containers.Public;
using Eagle._Interfaces.Public;

namespace Eagle._Interfaces.Private
{
    //
    // NOTE: This interface is currently private; however, it may be "promoted"
    //       to public at some point.
    //
    [ObjectId("198a1ed1-9f54-46e5-8dea-9e7e7f832673")]
    internal interface IDebugger : IHaveInterpreter
    {
        int SuspendCount { get; set; }
        bool Enabled { get; set; }
        int Active { get; set; }
        bool SingleStep { get; set; }

#if BREAKPOINTS
        bool BreakOnToken { get; set; }
#endif

        bool BreakOnExecute { get; set; }
        bool BreakOnCancel { get; set; }
        bool BreakOnError { get; set; }
        bool BreakOnReturn { get; set; }
        bool BreakOnTest { get; set; }
        bool BreakOnExit { get; set; }
        long Steps { get; set; }
        BreakpointType Types { get; set; }

#if BREAKPOINTS
        BreakpointDictionary Breakpoints { get; set; }
#endif

        string Command { get; set; }
        Result Result { get; set; }

        int SetActive(bool active); /* increase or decrease active debugger eval count */
        long NextStep(); /* advance the step counter */

#if BREAKPOINTS
        ReturnCode GetBreakpointList(string pattern, bool noCase,
            ref IStringList list, ref Result error);

        ReturnCode MatchBreakpoint(IScriptLocation location, ref bool match);
        ReturnCode MatchBreakpoint(IScriptLocation location, ref bool match,
            ref Result error);

        ReturnCode ClearBreakpoint(IScriptLocation location, ref bool match,
            ref Result error);
        ReturnCode SetBreakpoint(IScriptLocation location, ref bool match,
            ref Result error);
#endif

        ReturnCode Initialize(ref Result error); /* initialize the debugger to its
                                                  * default state (i.e. enabled and
                                                  * inactive) */

        ReturnCode Reset(ref Result error); /* disables all debugging features
                                             * (i.e. run script at full speed) */

        ReturnCode Suspend(ref Result error); /* temporarily suspend stepping
                                               * through code */

        ReturnCode Resume(ref Result error); /* resume from temporary suspention
                                              * of stepping through code */

        //
        // TODO: Change these to use the IInterpreter type.
        //
        ReturnCode Watch(Interpreter interpreter, ReturnCode code,
            BreakpointType breakpointType, string breakpointName,
            IToken token, ITraceInfo traceInfo, EngineFlags engineFlags,
            SubstitutionFlags substitutionFlags, EventFlags eventFlags,
            ExpressionFlags expressionFlags, HeaderFlags headerFlags,
            ref bool exit, ref Result result);

        ReturnCode Break(Interpreter interpreter, ReturnCode code,
            BreakpointType breakpointType, string breakpointName,
            IToken token, ITraceInfo traceInfo, EngineFlags engineFlags,
            SubstitutionFlags substitutionFlags, EventFlags eventFlags,
            ExpressionFlags expressionFlags, HeaderFlags headerFlags,
            IClientData clientData, ArgumentList arguments, ref bool exit,
            ref Result result);
    }
}
